/*******************************************************************************

	Program Name: CAP1114.C Version 0.2
	Copyright (C) 2013 Standard Microsystems Corporation
	All rights reverved.							

	This is sample program of CAP1114 for APP-ESS13-2 EVK


	Reference:																
		SMSC data sheet CAP1114 DS Rev. 1.1 (04-06-10).pdf

===============================================================================
Revision History:

Date
Version   	who  Comment
----------  --- --------------------------------------------------------------
2013/11/16	jc	.Initial version 
V0.1			
V0.2		    .Add proximity Demo
*******************************************************************************/

#include <xc.h>
#include <stdint.h>
#include "IIC.h"
#include "Main.h"

#define CAPADDRESS 		0x50			// CAPxxxx's slave address

extern uint8_t 	bG5mSCnt;

uint8_t	bUpDown = 0, bKStatus = 0; 	// Up/Down and button status 
uint8_t	bPromixity = 0;				// flag of proximity
uint8_t	b1S = 0xff;					// time counter for no touch


/*******************************************************************************
 marquee
*******************************************************************************/
void marquee( void ) 
{
	const  uint8_t  LEDScroll[] = {

//		0x80,	0x00,		// unlink
		0x94,	0x09,		// Direct_mode Rise_arte = Fall rate = 250 mS
		0x95,	0x00,		// off delay 0.0 S
 		0x70, 	0xff,		// LED/GPIO direction : output
		0x73,	0x01,
		0xEE,	50,
		0x73,	0x02,
		0xEE,	50,
		0x73,	0x04,
		0xEE,	50,
		0x73,	0x08,
		0xEE,	50,
		0x73,	0x10,
		0xEE,	50,
		0x73,	0x20,
		0xEE,	50,
		0x73,	0x40,
		0xEE,	200,
		0x73,	0x20,
		0xEE,	50,
		0x73,	0x10,
		0xEE,	50,
		0x73,	0x08,
		0xEE,	50,
		0x73,	0x04,
		0xEE,	50,
		0x73,	0x02,
		0xEE,	50,
		0x73,	0x01,
		0xEE,	50,
		0x73,	0x00,
		};


	uint8_t	bIdx;

	for ( bIdx = 0 ; bIdx < sizeof(LEDScroll)/sizeof(LEDScroll[0]) ; bIdx +=2)
	{
		if ( LEDScroll[bIdx] == 0xee )
		{
			bG5mSCnt = LEDScroll[bIdx+1];
			while( bG5mSCnt );
		}
		else
			WriteByte( CAPADDRESS, LEDScroll[bIdx], LEDScroll[bIdx+1]);
	}

}

/*******************************************************************************
 Initinalization of CAP 1114 & its alert pin interrupt if necessary.
*******************************************************************************/
void init_CAPxxxx( void ) 
{
	const  uint8_t  InitTable[] = {
// Sensors control
			0x1f, 0x3f,		// data sensitivity ( need to be adjusted for real system )
			0x20, 0x20,		// MAX_DUR_EN=0, RPT_EN = 0
			0x21, 0xff,		// enable all sensors
			0x27, 0xff,		// interrupt enable for all sensors
//			0x2A, 0x82,		// Multitouch enable for 1 key, and 4 for Slider
			0x2A, 0x0E,		// Multitouch disable
			0x2F, 0x0B,		// NEG=16
			0x30, 30,		// threshold of CS 1 ( need to be adjusted for real system )
			0x31, 40,		// threshold of CS 2 ( need to be adjusted for real system )
			0x32, 50,		// threshold of CS 3 ( need to be adjusted for real system )
			0x33, 60,		// threshold of CS 4 ( need to be adjusted for real system )
			0x34, 70,		// threshold of CS 5 ( need to be adjusted for real system )
			0x35, 80,		// threshold of CS 6 ( need to be adjusted for real system )
			0x36, 90,		// threshold of CS 7 ( need to be adjusted for real system )
			0x37, 40,		// threshold of slider ( need to be adjusted for real system )

//			0x42, 0x82,		// Enable CS1 for proximity channel
							// sensitivity ( need to be adjusted for real system )
// LEDs control
 			0x70, 0xff,		// LED/GPIO direction : output
 			0x71, 0x00,		// default LED/GPIO pin is open-drain output with external
                            // pull high
// 			0x73, 0x00,		// turn off LED 1-8 by default
// 			0x74, 0x00,		// turn off LEDs 9-11 by default
// 			0x75, 0x00,		// default polarity of LED 1-8 are inverted
// 			0x76, 0x00,		// default polarity of LED 9-10 are inverted

			0x81, 0x68,		// LED1=Direct mode LED2=LED3=Pulse 2 mode
			0x82, 0x3D,		// LED4=LED5=Pluse 1 mode, LED6=LED7= breathe mode
 			0x83, 0x06,		// LED 9-10 use Puluse 2 mode

			0x84, 0x10,		// Pulse 1 Period 512 mS
			0x85, 0x20,		// Pulse 2 Period 1024 mS
			0x86, 0x20,		// breathe period = 1024 mS
			0x88, 0x0b,		// Pulse 1 count=4, pulse 2 count = 2
			0x94, 0x09,		// Direct_mode Rise_arte = Fall rate = 0.25S DirectOff=0
			0x95, 0x00,		// Direct_mode Rise_arte = Fall rate = 0.25S DirectOff=0

			0x80, 0xFF,		// Link All LEDs

			0x26, 0xff,		// Force Recal.
			0x00, 0x00,		// reset INT. bit
			};


	uint8_t	bIdx;

        CAP_INT_PIN_TRIS=1;
        CAP_RESET_PIN=0;
        CAP_RESET_PIN_TRIS=0;

        //reset Cap1x14
        //RESET Filter Time tRST_FILT  >10ms
        CAP_RESET_PIN=1;
        bG5mSCnt = 3;
	while( bG5mSCnt );

        //RESET Pin release to fully active operation  tRST_ON Max =500ms
        CAP_RESET_PIN=0;
        bG5mSCnt = 100;
	while( bG5mSCnt );


	marquee( ) ;

	// Init. CAP1X14 registers
	for ( bIdx = 0 ; bIdx < sizeof(InitTable)/sizeof(InitTable[0]) ; bIdx +=2)
		WriteByte( CAPADDRESS, InitTable[bIdx], InitTable[bIdx+1]);

	if ( ReadByte( CAPADDRESS, 0xb1)  < 0x90 )	// enable CS1 as proximity channel
	{
		WriteByte( CAPADDRESS, 0x42, 0x83);
		WriteByte( CAPADDRESS, 0x1f, 0x2F);
		WriteByte( CAPADDRESS, 0x30, 70);
		WriteByte( CAPADDRESS, 0x20, 0x28);
		WriteByte( CAPADDRESS, 0x20, 0x28);
		bPromixity = 1;
	}

}


void GetKeyStatus( void ) 
{
	uint8_t	bTmp;

	// option 1. Detect alert pin's status or
	// option 2. test INT bit in Reg. 0x00
	bTmp=ReadByte(CAPADDRESS, 0);	// read register 0x00

	if ( bTmp & 0x01 )		// INT. asserts
	{
		// Read up/down status from bit 7,6, of reg. 0x03
		bUpDown = ReadByte(CAPADDRESS, 3);

		// Start to process buttons
		// clear INT status first
		WriteByte(CAPADDRESS, 0, 0 ); 
		bKStatus = ReadByte(CAPADDRESS, 3); // Read CS1-6's status from reg. 0x03
		bTmp = ReadByte(CAPADDRESS, 4); // Read CS7's status from reg. 0x04

		bKStatus &= 0x3f;
		if ( bTmp & 0x01 )	bKStatus |= 0x40;	// setup CS7's status
	} // INT. asserted
}

void PDemo(  )
{
	GetKeyStatus( );
	if ( bKStatus )
	{
		WriteByte( CAPADDRESS, 0x21, 0xff );
		WriteByte( CAPADDRESS, 0x75, 0xFe );
		WriteByte( CAPADDRESS, 0x76, 0x03 );
		WriteByte( CAPADDRESS, 0x83, 0x05 );
		b1S = 5;
		bG5mSCnt = 200;
	}
	else if ( b1S == 0xff )
	{
		WriteByte( CAPADDRESS, 0x21, 0x01 );
		WriteByte( CAPADDRESS, 0x75, 0x00 );
		WriteByte( CAPADDRESS, 0x76, 0x00 );
		WriteByte( CAPADDRESS, 0x83, 0x00 );
	}
	else if ( !bG5mSCnt )
	{	
		bG5mSCnt = 200;
		if ( --b1S == 0  )
		 	b1S = 0xff;
	}
}


